import {AudioSpeakerDetector} from 'phenix-web-sdk';    

// d3 package for Audio Visualization
import moment from 'moment';
import {scaleLinear, scaleBand, scalePow, scaleTime} from 'd3-scale';
import {axisBottom, axisLeft} from 'd3-axis';
import {select} from 'd3-selection';


class microphoneController {

    constructor(mediaFactory, $timeout) {

        var self = this;     
        this.title = "Microphone Device";
        self.devices = [];
        self.default = '';
        self.defaultAudioSource = '';
        self.stopped = false;
        self.startedRender = false;
         
        self.$onInit = function () {            
             // All available audio sources
            mediaFactory.audioSources().then(function(response){               
                self.devices = response.sources;
                self.defaultAudioSource = response.default;
                if(response.default) {
                    self.default = response.default; 
                } else {
                    self.default = response.sources[0]; 
                }
            
                //$timeout(function() {
                    mediaFactory.getMediaStream(self.defaultAudioSource, 'audio').then(function(response){

                        if(response) {
                            self.audioSpeakingDetector = new AudioSpeakerDetector([response]);              
                        
                            volumeDetector(self.audioSpeakingDetector, response);

                            mediaFactory.setPreMediaStream(response, 'audio');
                        }
                       
                    })
                //}, 2000)

            })             

        }  
        

        self.onSelect = function(data) {    
           
            let constraints = Object.assign({}, mediaFactory.deviceOptions());
            constraints.audio = { deviceId: data.id };  


            mediaFactory.getMediaStreamByConstraint(constraints, 'audio').then(function(response){
                let previousMediaStream = mediaFactory.previousMediaStream('audio');
               
                if(response) {
                    if( previousMediaStream !==  response ) {                     
                            
                        if(previousMediaStream) {
                           
                            disposeOfAudioContext.call(this);

                            const audioTrack = previousMediaStream.getAudioTracks()[0];
     
                            if (audioTrack) {
                                audioTrack.stop();
                                previousMediaStream.removeTrack(audioTrack);
                            }
                        }                        

                        self.audioSpeakingDetector = new AudioSpeakerDetector([response]);        
                
                        volumeDetector(self.audioSpeakingDetector, response);                       
                        mediaFactory.setPreMediaStream(response, 'audio');
                    } else {
                       
                    }

                    
                } else {

                    if(previousMediaStream) {
                        
                        disposeOfAudioContext.call(this);

                        const audioTrack = previousMediaStream.getAudioTracks()[0];
 
                        if (audioTrack) {
                            audioTrack.stop();
                            previousMediaStream.removeTrack(audioTrack);
                        }
                        
                        mediaFactory.setPreMediaStream(false, 'audio');
                    }
                }
               
            });            

        }        

        function stopRender() {
            self.stopped = true;
        }

        function disposeOfAudioContext(mediaStream) {
           
            if (self.startedRender) {
                stopRender();
                self.startedRender = false;
            }
            
            if (self.audioSpeakingDetector) {
                self.audioSpeakingDetector.stop();
                self.audioSpeakingDetector.dispose();
            }

            select('#audioSelectPreview').selectAll('*').remove();
        }

        function volumeDetector(audioSpeakingDetector, usermedia) {           
            let audioVolumeMeter = audioSpeakingDetector.getAudioVolumeMeter(usermedia);   
            plotData(audioVolumeMeter)    
        }
        
        function plotData(audioVolumeMeter) {

            self.startedRender = true;
            self.stopped = false;

            let textColor = {
                clipped: 'black',
                high: '#892B60',
                low: 'white',
                silence: '#CC3333',
            };
            
            let barColor = '#6699CC';

            let options = { 
                            element: '#audioSelectPreview',
                            title:  'Vol Level',
                            textColor: textColor,
                            barColor: barColor,
                        };

            const defaultMargin = {
                                    top: 0,
                                    left: 0,
                                    bottom: 0,
                                    right: 0,
                                };

            options.margin = options.margin || {};
            options.padding = options.padding || 0;

            Object.assign(options.margin, defaultMargin);

            const that = this;
            const element = select(options.element).node();
            const margin = {
                top: options.margin.top,
                left: options.margin.left,
                bottom: options.margin.bottom,
                right: options.margin.right,
            };
            
            const width = element.getBoundingClientRect().width - margin.left - margin.right;
            const height = element.getBoundingClientRect().height - margin.top - margin.bottom;
            const metrics = options.metrics || ['value'];
            const scale = {
            x: scaleBand()
                .range([0, width])
                .domain(metrics)
                .padding(options.padding),
            y: scalePow()
                .exponent(0.25)
                .domain([1, 0])
                .range([0, height])
            };

            const svg = select(options.element)
                .append('svg')
                .append('g')
                .attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');

            let lastValue = {
                date: Date.now(),
                value: 0.,
                smoothedValue: 0.,
                smoothedPeakValue: 0.,
                capped: 0,
            };

            audioVolumeMeter.onValue(function (update) {                            
                lastValue = update;
            });


            const renderStep = function renderStep() {
                if (self.stopped) {
                  return;
                }

                const textColor = lastValue.clipped > 0 ? options.textColor.clipped : lastValue.smoothedPeakValue > 0 ? lastValue.value > lastValue.smoothedValue ? options.textColor.high : options.textColor.low : options.textColor.silence;

                const barColor = options.barColor;

                const data = metrics.map(function (metric) {
                    return lastValue[metric];
                });

                svg.selectAll('*').remove();
                svg.attr('class', lastValue.clipped > 0 ? 'clipped' : lastValue.smoothedPeakValue > 0 ? lastValue.value > lastValue.smoothedValue ? 'high' : 'low' : 'silence');
                svg.selectAll('.bar')
                    .data(data)
                    .enter()
                    .append('rect')
                    .attr('class', 'bar')
                    .attr('x', function (d, i) {
                      return scale.x(metrics[i]);
                    })
                    .attr('width', scale.x.bandwidth())
                    .attr('y', scale.y)
                    .style('fill', barColor)
                    .attr('height', function (d) {
                      return height - scale.y(d);
                    });
                svg.append('text')
                    .attr('x', width / 2)
                    .attr('y', height / 2)
                    .attr('dy', 0.7 * height / 4)
                    .attr('font-size', height / 2)
                    .style('text-anchor', 'middle')
                    .style('text-align', 'center')
                    .style('fill', textColor)
                    .text(options.title);

                setTimeout(function () {
                    window.requestAnimationFrame(renderStep);
                }, 1000 / 24);
            };

            renderStep();

            const resize = function resize() {
                const width = element.getBoundingClientRect().width - margin.left - margin.right;
                const height = element.getBoundingClientRect().height - margin.top - margin.bottom;

                scale.x.range([0, width]);
                scale.y.range([0, height]);
            };

            window.addEventListener("resize", resize);
 
        }

    }
}

export default microphoneController;