import moment from 'moment';
import {scaleLinear, scaleBand, scalePow, scaleTime} from 'd3-scale';
import {axisBottom, axisLeft} from 'd3-axis';
import {select} from 'd3-selection';

const audioVolumeVisualizationModule = angular.module('audioVolumeVisualizationModule', [])
.factory('audioVolumeVisualization', ['$q', function ($q) {


    const defaultMargin = {
        top: 0,
        left: 0,
        bottom: 0,
        right: 0,
    };

    function startRender() {
        let textColor = {
                clipped: 'black',
                high: '#892B60',
                low: 'white',
                silence: '#CC3333',
            };
            
            let barColor = '#6699CC';

            let options = { 
                            element: '#audioSelectPreview',
                            title:  'Vol Level',
                            textColor: textColor,
                            barColor: barColor,
                        };

            const defaultMargin = {
                                    top: 0,
                                    left: 0,
                                    bottom: 0,
                                    right: 0,
                                };

            options.margin = options.margin || {};
            options.padding = options.padding || 0;

            Object.assign(options.margin, defaultMargin);

            const that = this;
            const element = select(options.element).node();
            const margin = {
                top: options.margin.top,
                left: options.margin.left,
                bottom: options.margin.bottom,
                right: options.margin.right,
            };
            
            const width = element.getBoundingClientRect().width - margin.left - margin.right;
            const height = element.getBoundingClientRect().height - margin.top - margin.bottom;
            const metrics = options.metrics || ['value'];
            const scale = {
            x: scaleBand()
                .range([0, width])
                .domain(metrics)
                .padding(options.padding),
            y: scalePow()
                .exponent(0.25)
                .domain([1, 0])
                .range([0, height])
            };

            const svg = select(options.element)
                .append('svg')
                .append('g')
                .attr('transform', 'translate(' + margin.left + ',' + margin.top + ')');

            let lastValue = {
                date: Date.now(),
                value: 0.,
                smoothedValue: 0.,
                smoothedPeakValue: 0.,
                capped: 0,
            };

            audioVolumeMeter.onValue(function (update) {
                console.log('update audio');              
                lastValue = update;
            });


            const renderStep = function renderStep() {
                // if (that.stopped) {
                //   return;
                // }

                const textColor = lastValue.clipped > 0 ? options.textColor.clipped : lastValue.smoothedPeakValue > 0 ? lastValue.value > lastValue.smoothedValue ? options.textColor.high : options.textColor.low : options.textColor.silence;

                const barColor = options.barColor;

                const data = metrics.map(function (metric) {
                    return lastValue[metric];
                });

                svg.selectAll('*').remove();
                svg.attr('class', lastValue.clipped > 0 ? 'clipped' : lastValue.smoothedPeakValue > 0 ? lastValue.value > lastValue.smoothedValue ? 'high' : 'low' : 'silence');
                svg.selectAll('.bar')
                    .data(data)
                    .enter()
                    .append('rect')
                    .attr('class', 'bar')
                    .attr('x', function (d, i) {
                      return scale.x(metrics[i]);
                    })
                    .attr('width', scale.x.bandwidth())
                    .attr('y', scale.y)
                    .style('fill', barColor)
                    .attr('height', function (d) {
                      return height - scale.y(d);
                    });
                svg.append('text')
                    .attr('x', width / 2)
                    .attr('y', height / 2)
                    .attr('dy', 0.7 * height / 4)
                    .attr('font-size', height / 2)
                    .style('text-anchor', 'middle')
                    .style('text-align', 'center')
                    .style('fill', textColor)
                    .text(options.title);

                setTimeout(function () {
                    window.requestAnimationFrame(renderStep);
                }, 1000 / 24);
            };

            renderStep();

            const resize = function resize() {
                const width = element.getBoundingClientRect().width - margin.left - margin.right;
                const height = element.getBoundingClientRect().height - margin.top - margin.bottom;

                scale.x.range([0, width]);
                scale.y.range([0, height]);
            };

            window.addEventListener("resize", resize);
    }




    

}]); 


export default audioVolumeVisualizationModule;    