import angular from 'angular';
import {RTC, PCast} from 'phenix-web-sdk'

const mediaFactoryModule = angular.module('mediaFactoryModule', [])
.factory('mediaFactory', ['$q', function ($q) {

    var self = this;

    self.pcast = new PCast();   
    
    self.options = {
                    audio: {},
                    video: {}
                };
    
    self.audioSources = [];
    self.videoSources = [];
    self.audioStream = '';
    self.videoStream = '';
    self.DefaultAudioSource = '';
    self.DefaultVideoSource = '';
    //self.previousConstraints = '';
    self.previousAudioStream = '';
    self.previousvideoStream = '';
    
    self.initialStream = '';


    const mapUserMediaConstraints = function (deviceOptions) {
        const constraints = {};

        if (deviceOptions.video) {
            // apply video quality constraints
            constraints.video = {
                height: { min: 360, ideal: 1080, max: 1080 },
                width: { min: 640, ideal: 1920, max: 1920 },
                frameRate: { ideal: 15, max: 30 },
            };

            if (deviceOptions.video.deviceId) {
                constraints.video.deviceId = { exact: deviceOptions.video.deviceId };
            }
        }

        if (deviceOptions.audio) {
            if (deviceOptions.audio.deviceId) {
                constraints.audio = {
                    deviceId: { exact: deviceOptions.audio.deviceId },
                };
            }   
        }
        
        return constraints;
    } 
    


    const getMediaStream = function (defaultSource, type_of_source) {

        let deferred = $q.defer();
        let constraints = Object.assign({}, self.options);

        if( type_of_source === 'audio' ) {          
            constraints.video = false;
            if(defaultSource) {
                self.options.audio.deviceId = defaultSource.id;
                constraints.audio.deviceId = defaultSource.id;
            }

            
        } else if ( type_of_source === 'video' ) {
            
            constraints.audio = false;
            if(defaultSource) {
                self.options.video.deviceId = defaultSource.id;
                constraints.video.deviceId = defaultSource.id;
            }
           
        } 
       
        constraints = mapUserMediaConstraints(constraints);      
        
        self.pcast.getUserMedia(constraints, function userMediaCallback(pcast, status, userMediaStream, error) {
                         
            if( status == 'ok' ) {             
                deferred.resolve(userMediaStream);
                
            } else {
                deferred.resolve(false);
            }   
            
        });      

        return deferred.promise;
    }
       

    const getMediaStreamByConstraint = function (given_constraints, type_of_source) {
        
        var deferred = $q.defer(); 

        if (given_constraints.audio.deviceId !== 'no-source' && given_constraints.audio.deviceId !== 'disabled' && Object.keys(given_constraints.audio).length > 0 ) {
            self.options.audio.deviceId = given_constraints.audio.deviceId;
        } else {
            given_constraints.audio = false;
            self.options.audio = {};
        }

        if (given_constraints.video.deviceId !== 'no-source' && given_constraints.video.deviceId !== 'disabled'  && Object.keys(given_constraints.video).length > 0 ) {
            self.options.video.deviceId = given_constraints.video.deviceId;
        } else {
            given_constraints.video = false;
            self.options.video = {}
        }

        if( type_of_source === 'audio' ) {
            given_constraints.video = false; 
        } else if ( type_of_source === 'video' ) {
            given_constraints.audio = false; 
        }

        console.log(self.options);
        console.log(given_constraints);

        // if (!given_constraints.video && !given_constraints.audio) {
        //     deferred.resolve(null);
        // } else {

            let constraints = mapUserMediaConstraints(given_constraints);

           // if (JSON.stringify(given_constraints) !== JSON.stringify(self.previousConstraints)) {

               // self.previousConstraints = given_constraints;
            
                self.pcast.getUserMedia(constraints, function userMediaCallback(pcast, status, userMediaStream, error) {
                                 
                    if( status == 'ok' ) {             
                        deferred.resolve(userMediaStream);
                        
                    } else {
                        deferred.resolve(false);
                    }              
                });
            // } else {
            //     deferred.reject(false);
            // }


        // }    

        return deferred.promise;
    }

    const getAudioSources = function () {
        var deferred = $q.defer(); 

        let initial = [{
                            "id": "disabled",
                            "label": "Select your microphone...", 
                            "kind": "audio" 
                        },
                        {
                            "id": "no-source",
                            "label": "No Audio", 
                            "kind": "audio"  
                        }];

        RTC.getSources((sources) => {
            let data = sources.filter((source) => source.kind === 'audio');  

            let response = {};
            if(data.length > 0) {                
                response.default = data[0];
            } else {    
                response.default = false;
            }

            response.sources = initial.concat(data);         
            deferred.resolve(response);  
        });

        return deferred.promise;
    }   

    const getVideoSources = function () {
        var deferred = $q.defer();
        let initial = [{
                            "id": "disabled",
                            "label": "Select your camera...", 
                            "kind": "video" 
                        },
                        {
                            "id": "no-source",
                            "label": "No Video", 
                            "kind": "video"  
                        }];

        RTC.getSources((sources) => {
            let data = sources.filter((source) => source.kind === 'video'); 
            let response = {};
            if(data.length > 0) {                
                response.default = data[0];
            } else {    
                response.default = false;
            }

            response.sources = initial.concat(data);
      
            deferred.resolve(response);
        });

        return deferred.promise;
    }   

    const setPreMediaStream = function(stream, type_of_source) {
        if( type_of_source === 'audio' ) {
            self.previousAudioStream = stream;
        } else if ( type_of_source === 'video' ) {
            self.previousVideoStream = stream;
        }        
    }

    const getDeviceOptions = function () {
        return self.options;
    }
    
    const getPreviousConstraints = function () {
        return self.previousConstraints;
    }

    const getPreviousMediaStream = function (type_of_source) {
        if( type_of_source === 'audio' ) {
            return self.previousAudioStream;
        } else if ( type_of_source === 'video' ) {
            return self.previousVideoStream;
        }       
    }

    var MediaService = {
        deviceOptions: getDeviceOptions,
        getMediaStream: getMediaStream,
        getMediaStreamByConstraint: getMediaStreamByConstraint, 
        audioSources: getAudioSources,
        videoSources: getVideoSources, 
        previousConstraints: getPreviousConstraints,
        previousMediaStream: getPreviousMediaStream, 
        setPreMediaStream: setPreMediaStream
    };

    return MediaService;

}]); 


export default mediaFactoryModule;    